import argparse
import torch
import torch.nn.parallel

from models import modules, net, resnet, densenet, senet
import numpy as np
import loaddata_demo as loaddata
import pdb

import matplotlib.image
import matplotlib.pyplot as plt
from PIL import Image

plt.set_cmap("jet")


def define_model(is_resnet, is_densenet, is_senet):
    if is_resnet:
        original_model = resnet.resnet50(pretrained = True)
        Encoder = modules.E_resnet(original_model) 
        model = net.model(Encoder, num_features=2048, block_channel = [256, 512, 1024, 2048])
    if is_densenet:
        original_model = densenet.densenet161(pretrained=True)
        Encoder = modules.E_densenet(original_model)
        model = net.model(Encoder, num_features=2208, block_channel = [192, 384, 1056, 2208])
    if is_senet:
        original_model = senet.senet154(pretrained='imagenet')
        Encoder = modules.E_senet(original_model)
        model = net.model(Encoder, num_features=2048, block_channel = [256, 512, 1024, 2048])

    return model
   

def main():
    groundtruth = Image.open('data/nyu2_test/00432_depth.png')
    groundtruth = np.array(groundtruth)
    matplotlib.image.imsave('data/demo/truth.png', groundtruth)
    
    model = define_model(is_resnet=False, is_densenet=False, is_senet=True)
    if torch.cuda.is_available():
        model = model.cuda()
        model.load_state_dict(torch.load('checkpoint_0.1_bilateral_5.pth'))
    else:
        model = model.to('cpu')
        model.load_state_dict(torch.load('checkpoint_0.1_bilateral_5.pth', map_location=torch.device('cpu')))
    model.eval()

    nyu2_loader = loaddata.readNyu2('data/nyu2_test/00432_colors.png')
  
    test(nyu2_loader, model)


def test(nyu2_loader, model):
    for i, image in enumerate(nyu2_loader):
        print(image.shape)
        plt.imshow(image[0,:,:,:].permute(1,2,0))
        plt.axis('off')
        plt.savefig('data/demo/noiseimage.png')

        if torch.cuda.is_available():     
            image = torch.autograd.Variable(image, volatile=True).cuda()
        else:
            image = torch.autograd.Variable(image, volatile=True).to('cpu')
        out = model(image)
        
        matplotlib.image.imsave('data/demo/out.png', out.view(out.size(2),out.size(3)).data.cpu().numpy())

        

if __name__ == '__main__':
    main()
