from scipy.ndimage import median_filter
import numpy as np
from skimage.filters import gaussian

def fspecial_gaussian_2d(size, sigma):
    kernel = np.zeros(tuple(size))
    kernel[size[0]//2, size[1]//2] = 1
    kernel = gaussian(kernel, sigma)
    return kernel/np.sum(kernel)

def bilateral2d(img, radius, sigma, sigmaIntensity):
    pad = radius
    # Initialize filtered image to 0
    out = np.zeros_like(img)

    # Pad image to reduce boundary artifacts
    imgPad = np.pad(img, pad)

    # Smoothing kernel, gaussian with standard deviation sigma
    # and size (2*radius+1, 2*radius+1)
    filtSize = (2*radius + 1, 2*radius + 1)
    spatialKernel = fspecial_gaussian_2d(filtSize, sigma)
    for y in range(img.shape[0]):
        for x in range(img.shape[1]):
            centerVal = imgPad[y+pad, x+pad] # Careful of padding amount!

            # Go over a window of size (2*radius + 1) around the current pixel,
            # compute weights, sum the weighted intensity.
            # Don't forget to normalize by the sum of the weights used.

            xPrime = imgPad[y+pad-radius:y+pad+radius+1, x+pad-radius:x+pad+radius+1]
            distanceOfIntensity = np.exp(-((xPrime - centerVal) ** 2) / (2*sigmaIntensity**2))
            num = np.sum(xPrime * distanceOfIntensity * spatialKernel)
            denomnum = np.sum(distanceOfIntensity * spatialKernel)
            out[y, x] = num/denomnum

    return out


def denoise_img(method, img):
	filtSizes = 5
	sigmas = 1
	sigmaIntensity =  0.5
	if(method == "median"):
		denoised_img = np.zeros_like(img)
		for channel in [0, 1, 2]:
			denoised_img[..., channel] = median_filter(img[..., channel], size=5) 
	else:
		denoised_img = np.zeros_like(img)
		for channel in [0, 1, 2]:
			denoised_img[..., channel] = bilateral2d(img[..., channel],
                                                  radius=int(sigmas),
                                                  sigma=sigmas,
                                                  sigmaIntensity=sigmaIntensity)
	return denoised_img

